package de.w665.sharepulse.db;

import com.rethinkdb.RethinkDB;
import com.rethinkdb.gen.exc.ReqlOpFailedError;
import com.rethinkdb.net.Connection;
import com.rethinkdb.net.Result;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

@Slf4j
@Service
public class RethinkDBService {
    private static final RethinkDB r = RethinkDB.r;
    private Connection connection;
    private final RethinkDBConfig config;

    public RethinkDBService(RethinkDBConfig config) {
        this.config = config;
    }

    @PostConstruct
    public void initialize() {
        connection = r.connection().hostname(config.getHost()).port(config.getPort()).connect();
        log.info("Connected to RethinkDB at " + config.getHost() + ":" + config.getPort() + " on database " + config.getDatabase());

        //rethinkdb check if database exists

        /*Result<Object> result = r.dbList().run(connection);
        List<String> databases = new ArrayList<>();

        Object db = result.first();
        ArrayList<String> dbList = (ArrayList<String>) db;
        databases.addAll(dbList);

        if (!databases.contains("sharepulse")) {
            r.dbCreate("sharepulse").run(connection);
            log.info("Database 'sharepulse' created successfully.");
        } else {
            log.info("Database 'sharepulse' already exists. No action needed.");
        }*/


        try {
            r.dbCreate(config.getDatabase()).run(connection).stream();
            log.info("Database " + config.getDatabase() + " created");
        } catch (ReqlOpFailedError e) {
            log.info("Database " + config.getDatabase() + " already exists. Error: " + e.getClass().getSimpleName());
        }



        // Additional initialization, like ensuring the database and tables exist
    }

    @PreDestroy
    public void close() {
        if (connection != null) {
            connection.close();
        }
    }

    // Example method to insert data into a specific table
    public void insertData(String tableName, Object data) {
        r.db(config.getDatabase()).table(tableName).insert(data).run(connection);
    }
}
