package de.w665.sharepulse.service;

import de.w665.sharepulse.model.FileUpload;
import jakarta.annotation.PostConstruct;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Date;

@Slf4j
@Service
public class FileService {

    private final FileIdService fileIdService;

    @Value("${sharepulse.temp-filestore-path}")
    private String tempDirPath;

    @Autowired
    public FileService(FileIdService fileIdService) {
        this.fileIdService = fileIdService;
    }

    public void processUploadedFile(MultipartFile file, String uploaderIp) {

        String fileId = fileIdService.generateNewUniqueId();

        FileUpload fileUpload = new FileUpload();
        fileUpload.setFileId(fileId);
        fileUpload.setFileName(file.getOriginalFilename());
        fileUpload.setFileSize(file.getSize());
        fileUpload.setSingleDownload(false);
        fileUpload.setUploadDate(new Date());
        fileUpload.setUploadedByIpAddress(uploaderIp);
        fileUpload.setDownloadCount(0);
        fileUpload.setFileDescription("fileDescription");

        System.out.println(fileUpload.toString());


        // TODO: rename file to fileID
        Path path = Paths.get(getTempDirPath() + File.separator + fileId);
        try {
            Files.write(path, file.getBytes());
            log.debug("File " + file.getOriginalFilename() + " written to " + path.getFileName());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // create a folder
    @PostConstruct
    public void createFolder() {
        Path path = Paths.get(getTempDirPath());
        log.debug("Creating directory " + path.getFileName());
        try {
            if (!Files.exists(path)) {
                Files.createDirectory(path);
                log.debug("Directory created");
            } else {
                log.debug("Directory already exists");
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private String getTempDirPath() {
        return System.getProperty("user.dir") + tempDirPath;
    }
}
